% script for an estimation of Lyapunov exponents of the Mackey-Glass (tau=17) equation, 
% comparing two simulation methods: the one from the Science manuscript
% (index 1) and the Matlab delay diff. equation solver dde23, with
% high-precision options set.

% obtained results are pasted in at end

% uses files MGDDEquation, disturbedseed, continuateSim1,
% generateSeedFromSample, randvecFineGrained

% compute a long MG trajectory longSample which loops around the attractor
% about 100 times. It will later be used in portions of length 17 to give
% seeds for Lyapunov exponent estimations. Highest precision is not needed
% here because these portions will be disturbed anyway.

global longSample;
longSampleLength = 3000;
longSample = dde23(@MGDDEquation,[17],1.2*ones(1,1),[0,longSampleLength]);


% Initialization of two sums of log ratios. Each summand to logRatioSum1 is
% the obtained as follows: 
% 1. compute a 317 step continuation of a "clean" section of the MG attractor.
%    Use the MG simulation method used in the Science submission.
% 2. take from that continuation an early small subsection A and a late subsection B
% 3. compute a 317 step continuation of a disturbed version of the clean seed 
%    used in 1, again using the method from the Science submission.
% 4. take from that continuation an early small subsection A' and a late subsection B'
% 5. estimate the rate of divergence by norm(B-B')/norm(A-A') and take the log of that
% The summands to logRatioSum2 are computed in the same way and from the
% same clean/disturbed seeds, but using the Matlab delay differential
% equation solver dde23.

logRatioSum1 = 0;
logRatioSum2 = 0;


% defining the boundaries of nearDist and farDist
ndStart = 101;
ndEnd = 117;
fdStart = 301;
fdEnd = 317;

% defining the fineness of the MG simulation method used for the Science
% submission
incrementsperUnit = 10;

% the big loop goes through longSample, using successive segments of length
% 17 as seeds for computing trajectory divergence from that seed onwards,
% computing divergence ratios for both my Science submission simulation
% method and the Matlab dde23 method (the latter is computed first inside
% the loop)

global nstart;
trials = 0;
for nstart = 300:17:longSampleLength - fdEnd - 1
    trials = trials + 1; % a counter for this loop
    
    
    % cut out from longSample a sniplet of length 17 time units, starting at nstart, and use it as
    % seed for another call to dde23, which continuates this seed for 150 time
    % units, generating shortSample2a. This time, set options for high-precision
    % solution. shortSample2a is a row vector of length 150
    
    
    global cleanSeedFun;
    cleanSeedFun = @generateSeedFromSample;
    
    options = ddeset('AbsTol', 1e-15, 'MaxStep', 1.0);
    shortSample2a = dde23(@MGDDEquation,[17], cleanSeedFun, [0,fdEnd], options);
    shortSample2a = deval(shortSample2a,1:fdEnd); % row vector
    
    
    % compute another sample shortSample2b like shortSample2a, but from a
    % disturbed version of the seed trajectory used for the latter.
    global randvec;
    randvec = 0.0000001 * (rand(17,1) - 0.5);
    disturbedseedFun = @disturbedseed;
    shortSample2b = dde23(@MGDDEquation,[17], disturbedseedFun, [0,fdEnd], options); 
    shortSample2b = deval(shortSample2b,1:fdEnd); % row vector
    
    neardistance2 = sqrt(sum((shortSample2a(ndStart:ndEnd) - shortSample2b(ndStart:ndEnd)).^ 2));
    fardistance2 = sqrt(sum((shortSample2a(fdStart:fdEnd) - shortSample2b(fdStart:fdEnd)).^ 2));
    
    
    logRatioSum2 = logRatioSum2 + log(fardistance2/neardistance2);
    
    
    % now compute similar neardistance1 and fardistance1 using the discrete
    % approximation from Science submission
    
    % the undisturbed case
    seed  = (deval(longSample, nstart-17:1/incrementsperUnit:nstart))';
    shortSample1a = (continuateSim1(seed, fdEnd, incrementsperUnit))'; % row vector
    
    % the disturbed case
    seedDisturbed  = (deval(longSample, nstart-17:1/incrementsperUnit:nstart))' ...
        + randvecFineGrained(randvec, incrementsperUnit);
    shortSample1b = (continuateSim1(seedDisturbed, fdEnd, incrementsperUnit))'; % row vector
    
    neardistance1 = sqrt(sum((shortSample1a(ndStart:ndEnd) - shortSample1b(ndStart:ndEnd)).^ 2));
    fardistance1 = sqrt(sum((shortSample1a(fdStart:fdEnd) - shortSample1b(fdStart:fdEnd)).^ 2));
    
    logRatioSum1 = logRatioSum1 + log(fardistance1/neardistance1);
    
end %of nstart loop

% computing (quasi-) LEs for the two simulation methods. LE1 for the
% Science submission method for simulating the attractor, LE2 for the
% Matlab solver dde23
timespan = fdStart - ndStart;
LE1 = logRatioSum1 / (timespan * trials)
LE2 = logRatioSum2 / (timespan * trials)

% the results obtained are LE1 = 0.0064, LE2 = 0.0059


