function [decorrOrderedSignalMat, corrVec, V, IX] = decorrelateSignals(signalMat, teacherVec)
% Standard use within ESN learning: decorrelate the state collection
% matrix.
% Input1: signalMat is a h by l matrix whose colums correspond to signals sampled over
% time (where rows correspond to time). 
% Input2: teacherVec is a column vector of the same length as columns in
% signalMat.
% Output1: decorrOrderedSignalMat is a matrix of same size as input matrix signalMat.  
% The columns of decorrOrderedSignalMat are mutually uncorrelated linear transforms
% of the columns of signalMat. Decorrelation is done via the Karhunen-Love transformation. 
% The columns in decorrOrderedSignalMat are ordered descending from left to right
% according to their absolute correlations with the teacherVec. That is, the
% first column of decorrOrderedSignalMat is the most useful for re-constructing
% teacherVec by linear regression, the second column is second most
% important, etc. 
% Output2: corrVec is a row vector of the same length as rows in signalMat.
% It contains the correlation coefficients of the columns of
% decorrOrderedSignalMat with teacherVec.
% Output3: V is the l by l decorrelation matrix used in the Karhunen-Love transform,
% that is signalMat * V gives a matrix of size h by l whose columns are
% decorrelated versions of the columns of signalMat
% Output4: IX is the permutation vector used to permute the columns of
% signalMat * V such that the resulting column-permuted matrix is ordered
% according to decreasing correlation coeffs with teacherVec

l = length(signalMat(1,:));
h = length(signalMat(:,1));
corrMat = signalMat' * signalMat;
[V,D] = eig(corrMat);
orthogonalSignalMat = signalMat * V;
orthogonalSignalMatNorm = orthogonalSignalMat;
for k = 1:l
    orthogonalSignalMatNorm(:,k) = orthogonalSignalMat(:,k) / norm(orthogonalSignalMat(:,k));
end
teacherVecNorm = teacherVec / norm(teacherVec);
teachCorrs = (orthogonalSignalMatNorm' * teacherVecNorm).^2;

% teachCorrsAbs now contains in its rows the  correlations between the 
%  normalized orthogonal signals of orthogonalSignalMat and the teacherVec signal

% Next we permute colums of orthogonalSignalMat into orthogonalSignalMatPermuted
% such that colums are in descending order of absolute correlation
% of the column signals with teach
[B, IX] = sort(- teachCorrs);
decorrOrderedSignalMat = orthogonalSignalMat(:,IX);
% compute corrVec
corrVec = (teachCorrs(IX,:))';
